#pragma once

#include <fstream>
#include<iostream>
#include<stdlib.h>
#include<vector>
#include<random>
#include<math.h>
#include<time.h>
#include<string>
#include "OptimizationProblemInterface.h"
#include "OptimizerInterface.h"

using namespace std;

class GeneticAlgorithm : public OptimizerInterface {
public:
	GeneticAlgorithm(OptimizationProblemInterface* op, int g_max, int np, double pc, double pm, double etac, double etam);
	virtual ~GeneticAlgorithm();
	vector<double> startAlgorithm();
private:
	int d;//Problem dimensionality
	int g_max;//Maximum generations
	int np;//Population size
	vector<double> d_max;//Upper bounds
	vector<double> d_min;//Lower bounds

	OptimizationProblemInterface* op;//Optimization problem
	vector<vector<double>> x;//Population with extended variables (one for the objective function and the other for the constraint sum)


	double scaledRand();//Return a random number in [0,1]
	void initializePopulation();//Generate a new population
	void crossover();//Parent matching
	void selection();//Sorting and pruning
	vector<double> poly_mutation(vector<double> ind);//PM
	vector<vector<double>> sbx(vector<double> parent1, vector<double> parent2);//SBX

	double pc;//Crossver probability
	double pm;//Mutation probability
	double etac;//Distribution index for SBX
	double etam;//Distribution index for PM

	//Sort the rows in a matrix based on an specific criteria
	void sortRows(std::vector<std::vector<double>>& matrix, int d, double ran);

	//Random Number Generator
	mt19937 rng;
	uniform_real_distribution<double> dist;
};


