#include "Obstacle.h"

// Constructor: Requires radius, initial x, and initial y coordinates, and the path type
Obstacle::Obstacle(double r, double x, double y, int pathType, double period, double amplitude) {
	this->r = r;
	this->x = x;
	this->y = y;
	this->pathType = pathType;
	this->period = period;
	this->amplitude = amplitude;
}

// Default obstacle
Obstacle::Obstacle() {
	this->r = 0;
	this->x = 0;
	this->y = 0;
	this->pathType = CIRCLE_PATH;
	this->period = 1;
	this->amplitude = 1;
}

// Get the position of the obstacle for a given time instant t
vector<double>  Obstacle::getPosition(double t) {
	switch (pathType)
	{
	case SIN_Y_PATH:
		return { x, y + amplitude * sin(period*t) };
		break;
	case COS_Y_PATH:
		return { x, y + amplitude * cos(period * t) };
		break;
	case SIN_X_PATH:
		return { x + amplitude * sin(period * t) , y};
		break;
	case COS_X_PATH:
		return { x + amplitude * cos(period * t) , y};
		break;
	case CIRCLE_PATH:
	default:
		return { x + amplitude * cos(period * t), y + amplitude * sin(period * t) };
		break;
	}
	
}

// Return the obstacle radius
double Obstacle::getR() {
	return r;
}

Obstacle::~Obstacle() {
}