#pragma once

#include <fstream>
#include<iostream>
#include<stdlib.h>
#include<vector>
#include<random>
#include<math.h>
#include<time.h>
#include<string>
#include "OptimizationProblemInterface.h"
#include "OptimizerInterface.h"

using namespace std;

class ParticleSwarmOptimization : public OptimizerInterface {
public:
	ParticleSwarmOptimization(OptimizationProblemInterface *op, int g_max, int np, double c1, double c2, double w_min, double w_max);
	virtual ~ParticleSwarmOptimization();
	vector<double> startAlgorithm();
private:
	int d;//Number of design variables
	vector<double> d_max;//Upper bounds
	vector<double> d_min;//Lower bounds

	int g_max;//Maximum iteration
	int np;//Swarm size
	double w_max;//Maximum inertia weight
	double w_min;//Minimum inertia weight
	double c1;//Local knowledge constant
	double c2;//Global knowledge constant

	OptimizationProblemInterface *op;//Optimization problem
	vector<vector<double>> x;//Swarm positions
	vector<vector<double>> x_best;//Personal best positions
	vector<vector<double>> x_dot;//Swarm velocities
	vector<double> x_best_swarm;//Global best position

	vector<double> f_x;//Swarm objective function values
	vector<double> f_x_best;//Personal best objective function values
	double f_x_best_swarm;//Global best objective function value

	vector<int> g_x;//Swarm constraints sum
	vector<int> g_x_best;//Personal best constraints sum
	double g_x_best_swarm;//Global best constraints sum

	void initializePopulation();//Generate a new swarm
	void initializeSwarmBest();//Search for the global best in the whole swarm
	double scaledRand();//Return a random number in [0,1]

	//Random Number Generator
	mt19937 rng;
	uniform_real_distribution<double> dist;
};

